`timescale 1ns/1ps

module testbench_lb;

// Inputs
reg flaga;
reg flagb;
reg flagc; //partial flag for IN EP
reg flagd; //full flag
reg clk;

reg forced_lock =0;
reg  reset_from_fx3;
reg  [2:0] mode;

// Outputs
wire [1:0] faddr1;
wire slrd1;
wire slwr1;
wire sloe1;
wire slcs1;
wire pktend1;
wire [3:0] GPIO_LED;

wire clk_out;
wire [1:0]  PMODE;  

// Bidirs
wire [31:0] fdata1;
wire [31:0] fdata2;
reg  [31:0] fdata_out;
reg  [31:0] fdata_in;

// Instantiate the Unit Under Test (UUT)
slaveFIFO2b_fpga_top slavefifo_if (
	.reset_in_(reset_from_fx3),            //input reset active low
	.clk(clk),                  //input clp 27 Mhz
	.fdata(fdata1),  
	.faddr(faddr1),                //output fifo address  
	.slrd(slrd1),                 //output read select
	.slwr(slwr1),                 //output write select
	.flaga(flaga),
	.flagb(flagb),
	.flagc(flagc),
	.flagd(flagd),
	.sloe(sloe1),                //output output enable select
	.clk_out(clk_out),             //output clk 100 Mhz and 180 phase shift
	.slcs(slcs1),                //output chip select
	.pktend(pktend1),              //output pkt end
	.mode_p(mode),
	.PMODE(PMODE),
	.RESET(RESET),
    .GPIO_LED(GPIO_LED),
    .forced_lock(forced_lock)
);

initial // Clock generator
begin
    clk = 0;
forever #18.518 clk = !clk;
end

initial begin
	mode  =3'd1;
	reset_from_fx3 = 1'b0;
	#100
	reset_from_fx3 = 1'b1;
    
    // Hack to simulate the PLL being locked in simulation 
	#600  forced_lock = 1'b1;
        
	#10000
   
    $finish();
end


assign fdata1 = (mode == 1) ? 32'hz : fdata_out;


// GPIF State machine 

parameter [2:0] FX3_RESET      = 3'd0;
parameter [2:0] FX3_IDLE       = 3'd1;
parameter [2:0] FX3_READ       = 3'd2;
parameter [2:0] FX3_WRITE      = 3'd3;
parameter [2:0] FX3_SHORT_PKT  = 3'd4;
parameter [2:0] ZLP            = 3'd5;
parameter [2:0] DSS_STATE      = 3'd6;

reg [2:0] next_gpif_state=FX3_RESET;
reg [2:0] current_gpif_state=FX3_RESET;

always @(posedge clk_out, negedge reset_from_fx3)begin
    if(!reset_from_fx3)begin 
        current_gpif_state <= FX3_RESET;
    end else begin
        current_gpif_state <= next_gpif_state;
    end 
end

// GPIF State machine combo 
always @(*)begin
    next_gpif_state = current_gpif_state;
    case(current_gpif_state)
        FX3_RESET:begin
            next_gpif_state = FX3_IDLE;
        end    
        FX3_IDLE:begin
            if (!slwr1 & !slcs1 & pktend1) begin 
               next_gpif_state = FX3_WRITE; 
            end else begin
                next_gpif_state = FX3_IDLE; 
            end     
        end    
        FX3_READ:begin
        end    
        FX3_WRITE:begin
            if ( ( pktend1 & slwr1 ) | slcs1 ) begin 
               next_gpif_state = FX3_IDLE; 
            end else begin
               next_gpif_state = FX3_WRITE; 
            end     
        end    
        FX3_SHORT_PKT:begin
        end    
        ZLP:begin
        end    
        DSS_STATE:begin
        end    

    endcase
end

// Sate machine output 
wire data_ready;
assign data_ready = (current_gpif_state == FX3_WRITE) ? 1'b1 : 1'b0;


// DMA state machine 
parameter [1:0] DMA_IDLE      = 3'd0;
parameter [1:0] DMA_READY     = 3'd1;
parameter [1:0] DMA_SWITCHING = 3'd2;

reg [1:0] next_dma_state    = DMA_IDLE;
reg [1:0] current_dma_state = DMA_IDLE;

parameter [15:0] DMA_SIZE      = 20;
parameter [15:0] DMA_WATERMARK = 4;
reg [15:0] dma_idx;

parameter MAX_DMA_SWITCH_CNT = 10;
reg [15:0] dma_switch_cnt;


reg  got_first_word = 1'b0;
reg  [31:0] rcvd_word;
reg  [31:0] rcvd_word_prev;


always @(posedge clk_out, negedge reset_from_fx3)begin
    if(!reset_from_fx3)begin 
        current_dma_state <= DMA_IDLE;
        dma_idx <= 0;
    end else begin
        current_dma_state <= next_dma_state;
        case(current_dma_state)
            DMA_IDLE:begin
                ;
            end    
            DMA_READY:begin
                if ( data_ready ) begin 
                    dma_idx <= dma_idx + 1; 
                    rcvd_word <= fdata1;
                    rcvd_word_prev <= rcvd_word;
                    got_first_word <= 1'b1;
                    if ( got_first_word ) begin 
                        if ( rcvd_word[31:24] !=  (rcvd_word_prev[7:0] + 1) ) begin
//                        if ( rcvd_word !=  (rcvd_word_prev + 1) ) begin
                            $display("Mismatch detected %h != %h + 1", rcvd_word, rcvd_word_prev);
                            $finish;
                        end
                    end
                end                   
                dma_switch_cnt <= 0;
            end    
            DMA_SWITCHING:begin
                dma_idx  <= 0;
                dma_switch_cnt <= dma_switch_cnt + 1;
                // Check for error conditions 
                if ( !slwr1 ) begin
                    $display("SLWR detected low while DMA not ready ");
                    $finish;
                end 
            end    
        endcase
    end 
end


// GPIF State machine combo 
always @(*)begin
    next_dma_state = current_dma_state;
    case(current_dma_state)
        DMA_IDLE:begin
            next_dma_state = DMA_READY;
        end    
        DMA_READY:begin
            if( dma_idx == DMA_SIZE) begin 
               next_dma_state = DMA_SWITCHING; 
            end else begin 
                next_dma_state = DMA_READY; 
            end    
        end    
        DMA_SWITCHING:begin
            if( dma_switch_cnt == MAX_DMA_SWITCH_CNT) begin 
               next_dma_state = DMA_READY; 
            end else begin 
                next_dma_state = DMA_SWITCHING; 
            end    
        end    
    endcase
end


// Manage DMA Flags

parameter FLAGS_DELAY = 3;

wire flaga_0; // Not delayed versions of flags
wire flagb_0;

assign flaga_0 = (current_dma_state == DMA_READY) ? 1'b1 : 1'b0;
assign flagb_0 = (dma_idx + DMA_WATERMARK  < DMA_SIZE) ? 1'b1 : 1'b0;
   
reg [FLAGS_DELAY-1:0] flaga_sr;
reg [FLAGS_DELAY-1:0] flagb_sr;
      
wire flaga_n; // Flags delayed by the N cycles latency 
wire flagb_n;
assign flaga_n = flaga_sr[0];
assign flagb_n = flagb_sr[0];
                    
always @(posedge clk_out, negedge reset_from_fx3)begin
    if(!reset_from_fx3)begin 
        flaga_sr[0] <= 0;
        flagb_sr[0] <= 0;
    end else begin
        flaga_sr  <= {flaga_0, flaga_sr[FLAGS_DELAY-1:1]};
        flagb_sr  <= {flagb_0, flagb_sr[FLAGS_DELAY-1:1]};      
    end  
end

// Add propagation latency to the flags 
`define TCFLG 8

always @(posedge clk_out)
    #(`TCFLG) flaga = flaga_n;

always @(posedge clk_out)
    #(`TCFLG) flagb = flagb_n;


endmodule

