//`define USE_PLAIN_DATA_GEN

(* keep_hierarchy = "yes" *)
module slaveFIFO2b_streamIN(
	input  reset_,
	input  clk_100,
	input  stream_in_mode_selected,
	input  flaga_d,
	input  flagb_d,
	output slwr_streamIN_,
	output [31:0] data_out_stream_in
);


reg [3:0]current_stream_in_state;
reg [3:0]next_stream_in_state;
reg [31:0]data_gen_stream_in;

//parameters for StreamIN mode state machine
parameter [3:0] stream_in_idle                    = 4'd0;
parameter [3:0] stream_in_wait_flagb              = 4'd1;
parameter [3:0] stream_in_write                   = 4'd2;
parameter [3:0] stream_in_write_wr_delay          = 4'd3;
parameter [3:0] stream_in_wait_flaga              = 4'd4;

wire rd_en;
wire fifo_valid;

// Outputs driven by the state machine  
assign slwr_streamIN_ = fifo_valid & ((current_stream_in_state == stream_in_write) | (current_stream_in_state == stream_in_write_wr_delay)) ? 1'b0 : 1'b1;
assign rd_en = ((current_stream_in_state == stream_in_write) | (current_stream_in_state == stream_in_write_wr_delay)) ? 1'b1 : 1'b0;


//streamIN mode state machine
always @(posedge clk_100, negedge reset_)begin
	if(!reset_)begin 
		current_stream_in_state <= stream_in_idle;
	end else begin
		current_stream_in_state <= next_stream_in_state;
	end	
end

//StreamIN mode state machine combo
always @(*)begin
	next_stream_in_state = current_stream_in_state;
	case(current_stream_in_state)
	stream_in_idle:begin
		if((stream_in_mode_selected) & (fifo_valid == 1'b1))begin
			next_stream_in_state = stream_in_wait_flaga; 
		end else begin
			next_stream_in_state = stream_in_idle;
		end	
	end
	stream_in_wait_flaga:begin
		if((stream_in_mode_selected) & (flaga_d == 1'b1) )begin // FIXME Why do we have two states ??? 
			next_stream_in_state = stream_in_wait_flagb; 
		end else begin
			next_stream_in_state = stream_in_idle;
		end	
	end
	stream_in_wait_flagb :begin
		if (flagb_d == 1'b1)begin
			next_stream_in_state = stream_in_write; 
		end else begin
			next_stream_in_state = stream_in_wait_flagb; 
		end
	end
	stream_in_write:begin
		if((flagb_d == 1'b0) | (fifo_valid == 1'b0) )begin
			next_stream_in_state = stream_in_idle;
		end else begin
		 	next_stream_in_state = stream_in_write;
		end
	end
        stream_in_write_wr_delay:begin
			next_stream_in_state = stream_in_idle;
	end
	endcase
end

//data generator counter for Partial, ZLP, StreamIN modes

`ifdef USE_PLAIN_DATA_GEN
always @(posedge clk_100, negedge reset_)begin
	if(!reset_)begin 
		data_gen_stream_in <= 32'd0;
	end else if((slwr_streamIN_ == 1'b0) & (stream_in_mode_selected)) begin
		data_gen_stream_in <= data_gen_stream_in + 1;
	end else if (!stream_in_mode_selected) begin
		data_gen_stream_in <= 32'd0;
	end	
end

assign data_out_stream_in = data_gen_stream_in;
assign fifo_valid = 1'b1;

`else

//Byte data generator counter 
reg [7:0]byte_gen_stream_in;
always @(posedge clk_100, negedge reset_)begin
	if(!reset_)begin 
		byte_gen_stream_in <= 8'd0;
	end else begin 
		byte_gen_stream_in <= byte_gen_stream_in + 1;
	end	
end


fx3_fifo fifo (
  .rst(!reset_), // input rst
  
  .wr_clk(clk_100), // input wr_clk
  .rd_clk(clk_100), // input rd_clk
  
  // FIFO write port 
  .din(byte_gen_stream_in), // input [7 : 0] din
  .wr_en(reset_), // input wr_en
  
  // FIFO read port 
  .rd_en(rd_en), // input rd_en
  .dout(data_out_stream_in), // output [31 : 0] dout
  
  .full(),  // FIXME  add error processing // output full  
  .empty(), // FIXME add error processing  // output empty
  
  .valid(fifo_valid), // output valid
  .rd_data_count() // output [8 : 0] rd_data_count
);

`endif // USE_PLAIN_DATA_GEN

endmodule
